{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Import libraries"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "import ee\n",
    "import geemap"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Create an interactive map"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "Map = geemap.Map(center=[40, -100], zoom=4)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Add Earth Engine Python script"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "# Add Earth Engine dataset\n",
    "image = ee.Image(\"USGS/SRTMGL1_003\")\n",
    "\n",
    "#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n",
    "#  Chapter:      A2.5 Water Balance and Drought\n",
    "#  Checkpoint:   A25s2\n",
    "#  Author:       Ate Poortinga, Quyen Nguyen, Nyein Soe Thwal, Andréa Puzzi Nicolau\n",
    "#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n",
    "\n",
    "# Import the Lower Mekong boundary.\n",
    "mekongBasin = ee.FeatureCollection(\n",
    "    'projects/gee-book/assets/A2-5/lowerMekongBasin')\n",
    "\n",
    "Map.centerObject(mekongBasin, 6)\n",
    "\n",
    "classStruct = {\n",
    "    'unknown': {\n",
    "        'number': 0,\n",
    "        'color': '6f6f6f'\n",
    "    },\n",
    "    'surface water': {\n",
    "        'number': 1,\n",
    "        'color': 'aec3d4'\n",
    "    },\n",
    "    'snow and ice': {\n",
    "        'number': 2,\n",
    "        'color': 'b1f9ff'\n",
    "    },\n",
    "    'mangroves': {\n",
    "        'number': 3,\n",
    "        'color': '111149'\n",
    "    },\n",
    "    'flooded forest': {\n",
    "        'number': 4,\n",
    "        'color': '287463'\n",
    "    },\n",
    "    'Deciduous forest': {\n",
    "        'number': 5,\n",
    "        'color': '152106'\n",
    "    },\n",
    "    'Orchard or plantation forest': {\n",
    "        'number': 6,\n",
    "        'color': 'c3aa69'\n",
    "    },\n",
    "    'evergreen Broadleaf': {\n",
    "        'number': 7,\n",
    "        'color': '7db087'\n",
    "    },\n",
    "    'mixed forest': {\n",
    "        'number': 8,\n",
    "        'color': '387242'\n",
    "    },\n",
    "    'urban and built up': {\n",
    "        'number': 9,\n",
    "        'color': 'cc0013'\n",
    "    },\n",
    "    'cropland': {\n",
    "        'number': 10,\n",
    "        'color': '8dc33b'\n",
    "    },\n",
    "    'rice': {\n",
    "        'number': 11,\n",
    "        'color': 'ffff00'\n",
    "    },\n",
    "    'mining': {\n",
    "        'number': 12,\n",
    "        'color': 'cec2a5'\n",
    "    },\n",
    "    'barren': {\n",
    "        'number': 13,\n",
    "        'color': '674c06'\n",
    "    },\n",
    "    'wetlands': {\n",
    "        'number': 14,\n",
    "        'color': '3bc3b2'\n",
    "    },\n",
    "    'grassland': {\n",
    "        'number': 15,\n",
    "        'color': 'f4a460'\n",
    "    },\n",
    "    'shrubland': {\n",
    "        'number': 16,\n",
    "        'color': '800080'\n",
    "    },\n",
    "    'aquaculture': {\n",
    "        'number': 17,\n",
    "        'color': '51768e'\n",
    "    }\n",
    "}\n",
    "\n",
    "# Function to get a list of ids (keys) from a structure.\n",
    "def getIds(struct):\n",
    "    return Object.keys(struct)\n",
    "\n",
    "\n",
    "# Function to replace spaces with underscores in a list of strings.\n",
    "def cleanList(list):\n",
    "\n",
    "def func_ebb(name):\n",
    "        return name.replace(/\\s+/g, '_')\n",
    "\n",
    "    return list.map(func_ebb)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "# Function to get a list of column values from a structure.\n",
    "def getList(struct, column):\n",
    "\n",
    "def func_ycx(k):\n",
    "        value = struct[k][column]\n",
    "        return value\n",
    "\n",
    "    return Object.keys(struct).map(func_ycx)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "classNamesList = getIds(classStruct)\n",
    "probNames = cleanList(classNamesList)\n",
    "classNames = ee.List(classNamesList)\n",
    "classNumbers = getList(classStruct, 'number')\n",
    "paletteList = getList(classStruct, 'color')\n",
    "PALETTE = paletteList.join(',')\n",
    "\n",
    "# JSON dictionary that defines piechart colors based on the\n",
    "# landcover class palette.\n",
    "# https:#developers.google.com/chart/interactive/docs/gallery/piechart\n",
    "colors = []\n",
    "for i in range(0, paletteList.length, 1):\n",
    "    colors.push({\n",
    "        'color': '_'.replace('_', paletteList[i])\n",
    "    })\n",
    "\n",
    "\n",
    "# Set start and end years.\n",
    "startYear = 2010\n",
    "endYear = 2020\n",
    "\n",
    "# Create two date objects for start and end years.\n",
    "startDate = ee.Date.fromYMD(startYear, 1, 1)\n",
    "endDate = ee.Date.fromYMD(endYear + 1, 1, 1)\n",
    "\n",
    "# Make a list with years.\n",
    "years = ee.List.sequence(startYear, endYear)\n",
    "\n",
    "# Make a list with months.\n",
    "months = ee.List.sequence(1, 12)\n",
    "\n",
    "# Import the MOD16 dataset.\n",
    "mod16 = ee.ImageCollection('MODIS/006/MOD16A2').select('ET')\n",
    "\n",
    "# Filter for relevant time period.\n",
    "mod16 = mod16.filterDate(startDate, endDate)\n",
    "\n",
    "# Import the CHIRPS dataset.\n",
    "CHIRPS = ee.ImageCollection('UCSB-CHG/CHIRPS/PENTAD')\n",
    "\n",
    "# Filter for relevant time period.\n",
    "CHIRPS = CHIRPS.filterDate(startDate, endDate)\n",
    "\n",
    "landcover = ee.Image(\n",
    "        'projects/gee-book/assets/A2-5/RLCMSv3/Mekonglandcover2018') \\\n",
    "    .select('lc').clip(mekongBasin)\n",
    "\n",
    "lcVis = {\n",
    "    'palette': PALETTE,\n",
    "    'min': 0,\n",
    "    'max': classNamesList.length - 1\n",
    "}\n",
    "Map.addLayer(landcover, lcVis, '2018 Land Cover')\n",
    "\n",
    "# We apply a nested loop where we first iterate over\n",
    "# the relevant years and then iterate over the relevant\n",
    "# months. The function returns an image with P - ET\n",
    "# for each month. A flatten is applied to convert an\n",
    "# collection of collections into a single collection.\n",
    "waterBalance = ee.ImageCollection.fromImages(\n",
    "\n",
    "def func_cly(y):\n",
    "        return months.map(function(m) {\n",
    "\n",
    "            P = CHIRPS.filter(ee.Filter \\\n",
    "                    .calendarRange(y, y, 'year')) \\\n",
    "                .filter(ee.Filter.calendarRange(m, m,\n",
    "                    'month')) \\\n",
    "                .sum()\n",
    "\n",
    "            ET = mod16.filter(ee.Filter \\\n",
    "                    .calendarRange(y, y, 'year')) \\\n",
    "                .filter(ee.Filter.calendarRange(m, m,\n",
    "                    'month')) \\\n",
    "                .sum() \\\n",
    "                .multiply(0.1)\n",
    "\n",
    "            wb = P.subtract(ET).rename('wb')\n",
    "\n",
    "            return wb.addBands(P).addBands(ET).set(\n",
    "                    'year', y) \\\n",
    "                .set('month', m) \\\n",
    "                .set('system:time_start', ee.Date \\\n",
    "                    .fromYMD(y, m, 1))\n",
    "        })\n",
    "\n",
    "    years.map(func_cly\n",
    ").flatten()\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    ").flatten()\n",
    ")\n",
    "\n",
    "# Calculate mean monthly values.\n",
    "monthlyP = waterBalance.select('precipitation').mean()\n",
    "monthlyET = waterBalance.select('ET').mean()\n",
    "monthlyWB = waterBalance.select('wb').mean()\n",
    "\n",
    "# Apply reducer per land cover category.\n",
    "# We create binary map for each class and multiply by mean ET.\n",
    "# The results are stored in a feature with other properties of the class.\n",
    "# The function returns a feature which are stored in a feature collection.\n",
    "\n",
    "def func_vdu(nr):\n",
    "    lc = landcover.eq(ee.Number(nr))\n",
    "\n",
    "    P = monthlyP.multiply(lc)\n",
    "    et = monthlyET.multiply(lc)\n",
    "\n",
    "    pSum = P.reduceRegion({\n",
    "        'reducer': ee.Reducer.sum(),\n",
    "        'geometry': mekongBasin,\n",
    "        'scale': 500\n",
    "    }).get('precipitation')\n",
    "\n",
    "    etSum = et.reduceRegion({\n",
    "        'reducer': ee.Reducer.sum(),\n",
    "        'geometry': mekongBasin,\n",
    "        'scale': 500\n",
    "    }).get('ET')\n",
    "\n",
    "    return ee.Feature(None).set('et', etSum) \\\n",
    "        .set('p', pSum) \\\n",
    "        .set('class_name', classNames.get(nr)) \\\n",
    "        .set('palette', paletteList[nr]) \\\n",
    "        .set('class_number', nr)\n",
    "\n",
    "lcFc = ee.FeatureCollection(classNumbers.map(func_vdu\n",
    "))\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "))\n",
    "\n",
    "#  Create the chart.\n",
    "pChart = ui.Chart.feature.byFeature({\n",
    "        'features': lcFc,\n",
    "        'xProperty': 'class_name',\n",
    "        'yProperties': ['p', 'class_number']\n",
    "    }) \\\n",
    "    .setChartType('PieChart') \\\n",
    "    .setOptions({\n",
    "        'title': 'amount of P per landcover class',\n",
    "        'slices': colors,\n",
    "        'sliceVisibilityThreshold': 0 # Don't group small slices.\n",
    "    })\n",
    "\n",
    "# Display the chart.\n",
    "print(pChart)\n",
    "\n",
    "# Create the chart.\n",
    "etChart = ui.Chart.feature.byFeature({\n",
    "        'features': lcFc,\n",
    "        'xProperty': 'class_name',\n",
    "        'yProperties': ['et', 'class_number']\n",
    "    }) \\\n",
    "    .setChartType('PieChart') \\\n",
    "    .setOptions({\n",
    "        'title': 'amount of ET per landcover class',\n",
    "        'slices': colors,\n",
    "        'sliceVisibilityThreshold': 0 # Don't group small slices.\n",
    "    })\n",
    "\n",
    "# Display the chart\n",
    "print(etChart)\n",
    "\n",
    "# Create the panel for the legend items.\n",
    "legend = ui.Panel({\n",
    "    'style': {\n",
    "        'position': 'bottom-left',\n",
    "        'padding': '8px 15px'\n",
    "    }\n",
    "})\n",
    "\n",
    "# Create and add the legend title.\n",
    "legendTitle = ui.Label({\n",
    "    'value': 'Legend',\n",
    "    'style': {\n",
    "        'fontWeight': 'bold',\n",
    "        'fontSize': '18px',\n",
    "        'margin': '0 0 4px 0',\n",
    "        'padding': '0'\n",
    "    }\n",
    "})\n",
    "\n",
    "# Creates and styles 1 row of the legend.\n",
    "def makeRow(color, name):\n",
    "    # Create the label that is actually the colored box.\n",
    "    colorBox = ui.Label({\n",
    "        'style': {\n",
    "            'backgroundColor': '#' + color,\n",
    "            # Use padding to give the box height and width.\n",
    "            'padding': '8px',\n",
    "            'margin': '0 0 4px 0'\n",
    "        }\n",
    "    })\n",
    "\n",
    "    # Create the label filled with the description text.\n",
    "    description = ui.Label({\n",
    "        'value': name,\n",
    "        'style': {\n",
    "            'margin': '0 0 4px 6px'\n",
    "        }\n",
    "    })\n",
    "\n",
    "    return ui.Panel({\n",
    "        'widgets': [colorBox, description],\n",
    "        'layout': ui.Panel.Layout.Flow('horizontal')\n",
    "    })\n",
    "\n",
    "\n",
    "legend.add(legendTitle)\n",
    "for i in range(0, classNamesList.length, 1):\n",
    "    legend.add(makeRow(paletteList[i], classNamesList[i]))\n",
    "\n",
    "\n",
    "# Add the legend to the map.\n",
    "Map.add(legend)\n",
    "\n",
    "# -----------------------------------------------------------------------------------\n",
    "# CHECKPOINT\n",
    "# -----------------------------------------------------------------------------------"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Display the interactive map"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "Map"
   ]
  }
 ],
 "metadata": {
  "anaconda-cloud": {},
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.6.1"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
